@@ -5,7 +5,8 @@ from django.core.exceptions import ImproperlyConfigured
 
 TRANSLATION_FILES = tuple(getattr(settings, 'MODELTRANSLATION_TRANSLATION_FILES', ()))
 
-AVAILABLE_LANGUAGES = [l[0] for l in settings.LANGUAGES]
+AVAILABLE_LANGUAGES = list(getattr(settings, 'MODELTRANSLATION_LANGUAGES',
+                                   (l[0] for l in settings.LANGUAGES)))
 DEFAULT_LANGUAGE = getattr(settings, 'MODELTRANSLATION_DEFAULT_LANGUAGE', None)
 if DEFAULT_LANGUAGE and DEFAULT_LANGUAGE not in AVAILABLE_LANGUAGES:
     raise ImproperlyConfigured('MODELTRANSLATION_DEFAULT_LANGUAGE not in LANGUAGES setting.')
@@ -35,7 +36,7 @@ AUTO_POPULATE = getattr(settings, 'MODELTRANSLATION_AUTO_POPULATE', False)
 # By default we fallback to the default language
 FALLBACK_LANGUAGES = getattr(settings, 'MODELTRANSLATION_FALLBACK_LANGUAGES', (DEFAULT_LANGUAGE,))
 if isinstance(FALLBACK_LANGUAGES, (tuple, list)):
-    FALLBACK_LANGUAGES = {'default': FALLBACK_LANGUAGES}
+    FALLBACK_LANGUAGES = {'default': tuple(FALLBACK_LANGUAGES)}
 if 'default' not in FALLBACK_LANGUAGES:
     raise ImproperlyConfigured(
         'MODELTRANSLATION_FALLBACK_LANGUAGES does not contain "default" key.')
@@ -53,3 +54,10 @@ for key, value in FALLBACK_LANGUAGES.items():
 ENABLE_FALLBACKS = getattr(settings, 'MODELTRANSLATION_ENABLE_FALLBACKS', True)
 
 LOADDATA_RETAIN_LOCALE = getattr(settings, 'MODELTRANSLATION_LOADDATA_RETAIN_LOCALE', True)
+
+JQUERY_URL = getattr(settings, 'JQUERY_URL',
+                     '//ajax.googleapis.com/ajax/libs/jquery/1.11.1/jquery.min.js')
+JQUERY_UI_URL = getattr(settings, 'JQUERY_UI_URL',
+                        '//ajax.googleapis.com/ajax/libs/jqueryui/1.11.2/jquery-ui.min.js')
+JQUERY_MB_BROWSER_URL = getattr(settings, 'JQUERY_MB_BROWSER_URL',
+                                '//cdn.jsdelivr.net/jquery.mb.browser/0.1/jquery.mb.browser.min.js')
