@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from django.conf import settings
 from django.core import validators
 from django.db import models
 from django.utils import six
@@ -321,12 +322,6 @@ class RequiredModel(models.Model):
     req_en_reg = models.CharField(max_length=10)
 
 
-# ######### Decorated registration testing
-
-class DecoratedModel(models.Model):
-    title = models.CharField(ugettext_lazy('title'), max_length=255)
-
-
 # ######### Name collision registration testing
 
 class ConflictModel(models.Model):
@@ -383,8 +378,8 @@ class ModelX(AbstractModelX):
 
 
 class AbstractModelXY(models.Model):
-    model_x = models.ForeignKey('ModelX')
-    model_y = models.ForeignKey('ModelY')
+    model_x = models.ForeignKey('ModelX', on_delete=models.CASCADE)
+    model_y = models.ForeignKey('ModelY', on_delete=models.CASCADE)
 
     class Meta:
         abstract = True
@@ -409,3 +404,12 @@ class AbstractModelY(models.Model):
 
 class ModelY(AbstractModelY):
     pass
+
+# Non-abstract base models whos Manager is not allowed to be overwritten
+
+
+if "django.contrib.auth" in settings.INSTALLED_APPS:
+    from django.contrib.auth.models import Permission
+
+    class InheritedPermission(Permission):
+        translated_var = models.CharField(max_length=255)
